# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fms.proto
"""Generated protocol buffer code."""
from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)
from google.protobuf.internal import enum_type_wrapper

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="fms.proto",
    package="fitelio.ms",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\tfms.proto\x12\nfitelio.ms"\x95\x02\n\x07Message\x12&\n\x06method\x18\x01 \x01(\x0e\x32\x16.fitelio.ms.SendMethod\x12&\n\x08provider\x18\x02 \x01(\x0e\x32\x14.fitelio.ms.Provider\x12\x12\n\nrecipients\x18\x05 \x03(\t\x12\x0c\n\x04text\x18\x06 \x01(\t\x12\x0b\n\x03uid\x18\x08 \x01(\t\x12\x0b\n\x03tag\x18\t \x01(\t\x12\x12\n\nalpha_name\x18\x0b \x01(\t\x12\r\n\x05topic\x18\x0c \x01(\t\x12\r\n\x05title\x18\r \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x0e \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\x0f \x01(\t\x12\r\n\x05image\x18\x10 \x01(\t\x12\x12\n\nstart_time\x18\x15 \x01(\t\x12\x0b\n\x03ttl\x18\x16 \x01(\x05"6\n\x0eMessageRequest\x12$\n\x07message\x18\x01 \x01(\x0b\x32\x13.fitelio.ms.Message"&\n\x05\x45rror\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x03\x12\x0f\n\x07message\x18\x02 \x01(\t"S\n\x0cMessageReply\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x03\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x11\n\trecipient\x18\x03 \x01(\t\x12\x12\n\nmessage_id\x18\x04 \x01(\t"^\n\x0fMessageResponse\x12 \n\x05\x65rror\x18\x01 \x01(\x0b\x32\x11.fitelio.ms.Error\x12)\n\x07results\x18\x02 \x03(\x0b\x32\x18.fitelio.ms.MessageReply**\n\nSendMethod\x12\x08\n\x04push\x10\x00\x12\x07\n\x03sms\x10\x01\x12\t\n\x05viber\x10\x02*A\n\x08Provider\x12\x07\n\x03\x66\x63m\x10\x00\x12\x08\n\x04\x61pns\x10\x01\x12\x0c\n\x08turbosms\x10\x02\x12\n\n\x06twilio\x10\x03\x12\x08\n\x04gmsu\x10\x04\x32X\n\x13MessagingServiceAPI\x12\x41\n\x04Send\x12\x1a.fitelio.ms.MessageRequest\x1a\x1b.fitelio.ms.MessageResponse"\x00\x62\x06proto3',
)

_SENDMETHOD = _descriptor.EnumDescriptor(
    name="SendMethod",
    full_name="fitelio.ms.SendMethod",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="push",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="sms",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="viber",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=582,
    serialized_end=624,
)
_sym_db.RegisterEnumDescriptor(_SENDMETHOD)

SendMethod = enum_type_wrapper.EnumTypeWrapper(_SENDMETHOD)
_PROVIDER = _descriptor.EnumDescriptor(
    name="Provider",
    full_name="fitelio.ms.Provider",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="fcm",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="apns",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="turbosms",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="twilio",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="gmsu",
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=626,
    serialized_end=691,
)
_sym_db.RegisterEnumDescriptor(_PROVIDER)

Provider = enum_type_wrapper.EnumTypeWrapper(_PROVIDER)
push = 0
sms = 1
viber = 2
fcm = 0
apns = 1
turbosms = 2
twilio = 3
gmsu = 4


_MESSAGE = _descriptor.Descriptor(
    name="Message",
    full_name="fitelio.ms.Message",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="method",
            full_name="fitelio.ms.Message.method",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="provider",
            full_name="fitelio.ms.Message.provider",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="recipients",
            full_name="fitelio.ms.Message.recipients",
            index=2,
            number=5,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="text",
            full_name="fitelio.ms.Message.text",
            index=3,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="uid",
            full_name="fitelio.ms.Message.uid",
            index=4,
            number=8,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="tag",
            full_name="fitelio.ms.Message.tag",
            index=5,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="alpha_name",
            full_name="fitelio.ms.Message.alpha_name",
            index=6,
            number=11,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="topic",
            full_name="fitelio.ms.Message.topic",
            index=7,
            number=12,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="title",
            full_name="fitelio.ms.Message.title",
            index=8,
            number=13,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="data",
            full_name="fitelio.ms.Message.data",
            index=9,
            number=14,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="action",
            full_name="fitelio.ms.Message.action",
            index=10,
            number=15,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="image",
            full_name="fitelio.ms.Message.image",
            index=11,
            number=16,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="start_time",
            full_name="fitelio.ms.Message.start_time",
            index=12,
            number=21,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ttl",
            full_name="fitelio.ms.Message.ttl",
            index=13,
            number=22,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=26,
    serialized_end=303,
)


_MESSAGEREQUEST = _descriptor.Descriptor(
    name="MessageRequest",
    full_name="fitelio.ms.MessageRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="message",
            full_name="fitelio.ms.MessageRequest.message",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=305,
    serialized_end=359,
)


_ERROR = _descriptor.Descriptor(
    name="Error",
    full_name="fitelio.ms.Error",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="code",
            full_name="fitelio.ms.Error.code",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="message",
            full_name="fitelio.ms.Error.message",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=361,
    serialized_end=399,
)


_MESSAGEREPLY = _descriptor.Descriptor(
    name="MessageReply",
    full_name="fitelio.ms.MessageReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="code",
            full_name="fitelio.ms.MessageReply.code",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status",
            full_name="fitelio.ms.MessageReply.status",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="recipient",
            full_name="fitelio.ms.MessageReply.recipient",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="message_id",
            full_name="fitelio.ms.MessageReply.message_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=401,
    serialized_end=484,
)


_MESSAGERESPONSE = _descriptor.Descriptor(
    name="MessageResponse",
    full_name="fitelio.ms.MessageResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="error",
            full_name="fitelio.ms.MessageResponse.error",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="results",
            full_name="fitelio.ms.MessageResponse.results",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=486,
    serialized_end=580,
)

_MESSAGE.fields_by_name["method"].enum_type = _SENDMETHOD
_MESSAGE.fields_by_name["provider"].enum_type = _PROVIDER
_MESSAGEREQUEST.fields_by_name["message"].message_type = _MESSAGE
_MESSAGERESPONSE.fields_by_name["error"].message_type = _ERROR
_MESSAGERESPONSE.fields_by_name["results"].message_type = _MESSAGEREPLY
DESCRIPTOR.message_types_by_name["Message"] = _MESSAGE
DESCRIPTOR.message_types_by_name["MessageRequest"] = _MESSAGEREQUEST
DESCRIPTOR.message_types_by_name["Error"] = _ERROR
DESCRIPTOR.message_types_by_name["MessageReply"] = _MESSAGEREPLY
DESCRIPTOR.message_types_by_name["MessageResponse"] = _MESSAGERESPONSE
DESCRIPTOR.enum_types_by_name["SendMethod"] = _SENDMETHOD
DESCRIPTOR.enum_types_by_name["Provider"] = _PROVIDER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Message = _reflection.GeneratedProtocolMessageType(
    "Message",
    (_message.Message,),
    {
        "DESCRIPTOR": _MESSAGE,
        "__module__": "fms_pb2"
        # @@protoc_insertion_point(class_scope:fitelio.ms.Message)
    },
)
_sym_db.RegisterMessage(Message)

MessageRequest = _reflection.GeneratedProtocolMessageType(
    "MessageRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _MESSAGEREQUEST,
        "__module__": "fms_pb2"
        # @@protoc_insertion_point(class_scope:fitelio.ms.MessageRequest)
    },
)
_sym_db.RegisterMessage(MessageRequest)

Error = _reflection.GeneratedProtocolMessageType(
    "Error",
    (_message.Message,),
    {
        "DESCRIPTOR": _ERROR,
        "__module__": "fms_pb2"
        # @@protoc_insertion_point(class_scope:fitelio.ms.Error)
    },
)
_sym_db.RegisterMessage(Error)

MessageReply = _reflection.GeneratedProtocolMessageType(
    "MessageReply",
    (_message.Message,),
    {
        "DESCRIPTOR": _MESSAGEREPLY,
        "__module__": "fms_pb2"
        # @@protoc_insertion_point(class_scope:fitelio.ms.MessageReply)
    },
)
_sym_db.RegisterMessage(MessageReply)

MessageResponse = _reflection.GeneratedProtocolMessageType(
    "MessageResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _MESSAGERESPONSE,
        "__module__": "fms_pb2"
        # @@protoc_insertion_point(class_scope:fitelio.ms.MessageResponse)
    },
)
_sym_db.RegisterMessage(MessageResponse)


_MESSAGINGSERVICEAPI = _descriptor.ServiceDescriptor(
    name="MessagingServiceAPI",
    full_name="fitelio.ms.MessagingServiceAPI",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=693,
    serialized_end=781,
    methods=[
        _descriptor.MethodDescriptor(
            name="Send",
            full_name="fitelio.ms.MessagingServiceAPI.Send",
            index=0,
            containing_service=None,
            input_type=_MESSAGEREQUEST,
            output_type=_MESSAGERESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_MESSAGINGSERVICEAPI)

DESCRIPTOR.services_by_name["MessagingServiceAPI"] = _MESSAGINGSERVICEAPI

# @@protoc_insertion_point(module_scope)



import grpc
import structlog
from fms import settings
from fms.pb import fms_pb2 as pb, fms_pb2_grpc
from grpc_health.v1 import health_pb2, health_pb2_grpc

logger = structlog.getLogger(__name__)


def health_check():
    channel = grpc.insecure_channel(settings.MS_SERVER_URL)
    stub = health_pb2_grpc.HealthStub(channel)
    response = stub.Check(health_pb2.HealthCheckRequest(service="fms-server"), timeout=10)
    logger.info(f"response: {response}")


class FMSError(Exception):
    def __init__(self, code, message):
        self.code = code
        self.message = message


class FMSClient:
    def __init__(self, raise_exception=False, timeout=10):
        self.client = fms_pb2_grpc.MessagingServiceAPIStub(grpc.insecure_channel(settings.MS_SERVER_URL))
        self.raise_exception = raise_exception
        self.timeout = timeout
        self.log = logger

    @classmethod
    def _build_message(
        cls,
        recipients,
        text,
        uid="",
        tag="",
        method=None,
        provider=None,
        alpha_name="",
        topic=None,
        title="",
        action=None,
        data=None,
        image=None,
        start_time=None,
        ttl=30,
    ):

        if method:
            method = pb.SendMethod.Value(method)
        if provider:
            provider = pb.Provider.Value(provider)

        return pb.Message(
            recipients=recipients,
            text=text,
            uid=uid,
            tag=tag,
            method=method,
            provider=provider,
            alpha_name=alpha_name,
            topic=topic,
            title=title,
            action=action,
            data=data,
            image=image,
            start_time=start_time,
            ttl=ttl,
        )

    def _handle_error(self, message, code=""):
        self.log.error(code=code, status=message)
        if self.raise_exception:
            raise FMSError(code, message)
        return None

    def send_message(
        self,
        recipients,
        text,
        uid="",
        tag="",
        method=None,
        provider=None,
        alpha_name="",
        topic="",
        title="",
        action="",
        data=None,
        image=None,
        start_time=None,
        ttl=30,
    ):
        self.log = self.log.bind(
            message={
                str(recipients),
                text,
                uid,
                tag,
                method,
                provider,
                alpha_name,
                topic,
                title,
                action,
                data,
                image,
                start_time,
                ttl,
            }
        )

        message = self._build_message(
            recipients,
            text,
            uid=uid,
            tag=tag,
            method=method,
            provider=provider,
            alpha_name=alpha_name,
            topic=topic,
            title=title,
            action=action,
            data=data,
            image=image,
            start_time=start_time,
            ttl=ttl,
        )
        try:
            response = self.client.Send(pb.MessageRequest(message=message), timeout=self.timeout)
        except grpc.RpcError as e:
            return self._handle_error(f"RPC call failed: {e}")
        except Exception as e:
            return self._handle_error(f"RPC call failed: {e}")
        if response.error.code:
            return self._handle_error(message=response.error.message, code=response.error.code)

        result = [
            {"code": res.code, "status": res.status, "recipient": res.recipient, "message_id": res.message_id}
            for res in response.results
        ]
        self.log.info(result=result)
        return result


__all__ = ["FMSError", "FMSClient", "health_check"]
